@@ -93,6 +93,41 @@ class Graph:
     def __repr__(self):
         return f'<{self.__class__.__name__} with {len(self.nodes)} nodes, {len(self.edges)} edges>'
 
+    def _make_foreign_graph(self):
+        '''Translate this object into a foreign graph object for rendering'''
+        foreign = self._graph_cls()
+        foreign.attr('graph', **vars(self.attrs))
+
+        node_names = set()
+        for node in self.nodes:
+            if not hasattr(node.attrs, 'name') \
+            and hasattr(node.attrs, 'label'):
+                node.attrs.name = re.sub(r'\W', '', node.attrs.label)
+            while node.attrs.name in node_names:  # do not allow node name collisions
+                node.attrs.name += "_"
+            node_names.add(node.attrs.name)
+            foreign.node(**vars(node.attrs))
+        for edge in self.edges:
+            foreign.edge(
+                tail_name=edge.start.attrs.name,
+                head_name=edge.end.attrs.name,
+                **vars(edge.attrs),
+            )
+        return foreign
+
+    def _dot_foreign_graph(self, foreign):
+        '''Return dot lang source for foreign graph'''
+        return foreign.source
+
+    def _save_foreign_graph(self, foreign, filename, fileformat):
+        '''Render foreign graph to a file on disk'''
+        foreign.render(
+            filename=Path(filename).with_suffix(''), # backwards compatible with pypi/graphviz==0.16
+            format = fileformat,
+            cleanup=True,
+            view=False,
+        )
+
     def render(self, filename=None, fmt=None):
         '''
         Render graph
@@ -108,27 +143,10 @@ class Graph:
         if filename is None and fmt != 'dot':
             raise ValueError(f'cannot render {fmt} without a filename to save to')
 
-        gv = self._graph_cls()
-        gv.attr('graph', **vars(self.attrs))
-
-        node_names = set()
-        for node in self.nodes:
-            if not hasattr(node.attrs, 'name') \
-            and hasattr(node.attrs, 'label'):
-                node.attrs.name = re.sub(r'\W', '', node.attrs.label)
-            while node.attrs.name in node_names:  # do not allow node name collisions
-                node.attrs.name += "_"
-            node_names.add(node.attrs.name)
-            gv.node(**vars(node.attrs))
-        for edge in self.edges:
-            gv.edge(
-                tail_name=edge.start.attrs.name,
-                head_name=edge.end.attrs.name,
-                **vars(edge.attrs),
-            )
+        gv = self._make_foreign_graph()
 
         if fmt == 'dot' and filename is None:
-            return gv.source
+            return self._dot_foreign_graph(gv)
 
         output = Path(filename)
         if fmt is None:
@@ -136,15 +154,9 @@ class Graph:
         output.parent.mkdir(parents=True, exist_ok=True)
         if fmt == 'dot':
             with output.open('w') as f:
-                f.write(gv.source)
+                f.write(self._dot_foreign_graph(gv))
         else:
-            gv.format = fmt
-            gv.render(
-                outfile=filename,
-                filename=str(filename) + '.gv',
-                cleanup=True,
-                view=False,
-            )
+            self._save_foreign_graph(gv, filename, fmt)
 
     def node(self, cls=None, **attrs):
         '''Add new node to graph'''
